"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatError = exports.ingestParams = exports.parseParamKey = void 0;
const json_patch_1 = require("../json-patch");
const Value_1 = require("../reactive-rpc/common/messages/Value");
const caller_1 = require("../reactive-rpc/common/rpc/caller");
const PARAM_REGEX = /^([a-z]+)(\/.*)$/;
const parseParamKey = (key) => {
    const match = PARAM_REGEX.exec(key);
    if (!match)
        return [];
    const [, type, path] = match;
    return [type, path];
};
exports.parseParamKey = parseParamKey;
const ingestParams = (params, result) => {
    for (const key of Object.keys(params)) {
        const [type, path] = (0, exports.parseParamKey)(key);
        if (!type)
            continue;
        switch (type) {
            case 'j':
            case 'json': {
                const value = JSON.parse(params[key]);
                (0, json_patch_1.applyPatch)(result, [{ op: 'add', path: path, value }], { mutate: true });
                break;
            }
            case 'n':
            case 'num': {
                const value = Number(JSON.parse(params[key]));
                (0, json_patch_1.applyPatch)(result, [{ op: 'add', path: path, value }], { mutate: true });
                break;
            }
            case 's':
            case 'str': {
                const value = String(params[key]);
                (0, json_patch_1.applyPatch)(result, [{ op: 'add', path: path, value }], { mutate: true });
                break;
            }
            case 'b':
            case 'bool': {
                const value = Boolean(JSON.parse(params[key]));
                (0, json_patch_1.applyPatch)(result, [{ op: 'add', path: path, value }], { mutate: true });
                break;
            }
            case 'nil': {
                (0, json_patch_1.applyPatch)(result, [{ op: 'add', path: path, value: null }], { mutate: true });
                break;
            }
            case 'und': {
                (0, json_patch_1.applyPatch)(result, [{ op: 'add', path: path, value: undefined }], { mutate: true });
                break;
            }
            default:
                throw new Error(`Invalid param type: ${type}`);
        }
    }
};
exports.ingestParams = ingestParams;
const formatError = (err) => {
    if (err instanceof Value_1.Value)
        return (0, exports.formatError)(err.data);
    if (err instanceof caller_1.RpcError)
        return err.toJson();
    if (err instanceof Error)
        return { message: err.message, stack: err.stack };
    return err;
};
exports.formatError = formatError;
